@@ -123,7 +123,8 @@ struct ldap_info
 	gid_t gid;
 	int   timeout;
 	int   authbind;
-
+	int   deref;
+	int   tls;
 	int	authbind_ok;  /* not used? */
 };
 
@@ -351,11 +352,32 @@ static int authldap_read_config(struct ldap_info *ldap)
 
   ldap->timeout=5;
   p=0;
-  if (!read_env("LDAP_TIMEOUT", &p, "", 0, ""))
+  if (read_env("LDAP_TIMEOUT", &p, "", 0, "") && p)
+  {
+    sscanf(p,"%d",&ldap->timeout);
+  }
+
+  ldap->tls=0;
+  p=0;
+  if (read_env("LDAP_TLS", &p, "", 0, "") && p)
+  {
+    ldap->tls=atoi(p);
+  }
+
+  ldap->deref = LDAP_DEREF_NEVER; 
+  p=0;
+  if (!read_env("LDAP_DEREF", &p, "", 0, ""))
 	return (0);
   if (p)
   {
-    sscanf(p,"%d",&ldap->timeout);
+	if (!strcasecmp (p, "never"))
+	  ldap->deref = LDAP_DEREF_NEVER;
+	else if (!strcasecmp (p, "searching"))
+	  ldap->deref = LDAP_DEREF_SEARCHING;
+	else if (!strcasecmp (p, "finding"))
+	  ldap->deref = LDAP_DEREF_FINDING;
+	else if (!strcasecmp (p, "always"))
+	  ldap->deref = LDAP_DEREF_ALWAYS; 
   }
 
   return 1;
@@ -495,7 +517,67 @@ int     ldrc;
 		return (1);
 	}
 
+#if HAVE_LDAP_TLS
+	if (my_ldap.tls)
+	{
+		int version;
 
+		if (ldaperror(ldrc=ldap_get_option (my_ldap_fp,
+						    LDAP_OPT_PROTOCOL_VERSION,
+						    &version))
+		    != LDAP_SUCCESS)
+		{
+			const char *s=ldap_err2string(ldrc);
+
+#if	HAVE_SYSLOG_H
+			syslog(LOG_DAEMON|LOG_CRIT,
+			       "ldap_get_option failed: %s", s);
+#endif
+			authldapclose();
+			ldapconnfailure();
+			return (-1);
+		}
+
+		if (version < LDAP_VERSION3)
+		{
+			version = LDAP_VERSION3;
+			(void)ldap_set_option (my_ldap_fp,
+					       LDAP_OPT_PROTOCOL_VERSION,
+					       &version);
+		}
+
+		if (ldaperror(ldrc=ldap_start_tls_s(my_ldap_fp, NULL, NULL))
+		    != LDAP_SUCCESS)
+		{
+			const char *s=ldap_err2string(ldrc);
+
+#if	HAVE_SYSLOG_H
+			syslog(LOG_DAEMON|LOG_CRIT,
+			       "ldap_start_tls_s failed: %s", s);
+#endif
+			authldapclose();
+			ldapconnfailure();
+			return (-1);
+		}
+	}
+#endif
+
+#ifdef LDAP_OPT_DEREF
+
+	/* Set deferencing mode */
+	if (ldaperror(ldrc = ldap_set_option(my_ldap_fp, LDAP_OPT_DEREF,
+					 (void *) & my_ldap.deref)) != LDAP_SUCCESS)
+	  {
+		const char *s=ldap_err2string(ldrc);
+
+#if	HAVE_SYSLOG_H
+		syslog(LOG_DAEMON|LOG_CRIT, "ldap_set_option failed: %s", s);
+#endif
+		authldapclose();
+		ldapconnfailure();
+		return (-1);
+	  }
+#endif
 
   /* Bind to server */
 #if DEBUG_LDAP
@@ -518,6 +600,17 @@ int     ldrc;
 	return (0);
 }
 
+static int auth_ldap_do(const char *, const char *,
+			int (*)(struct authinfo *, void *),
+                        void *arg, const char *);
+
+int auth_ldap_changepw(const char *dummy, const char *user,
+		       const char *pass,
+		       const char *newpass)
+{
+	return auth_ldap_do(user, pass, NULL, NULL, newpass);
+}
+
 /*
  * Function: authldapcommon
  *   Get information from the LDAP server ($ldap) for this $user
@@ -533,48 +626,56 @@ int     ldrc;
  */
 
 int authldapcommon(const char *user, const char *pass,
-        int (*callback)(struct authinfo *, void *),
-                        void *arg)
+		   int (*callback)(struct authinfo *, void *),
+		   void *arg)
 {
-  const char *attributes[10], *ldap_attributes[10];
+	return (auth_ldap_do(user, pass, callback, arg, NULL));
+}
+
+static int auth_ldap_do(const char *user, const char *pass,
+			int (*callback)(struct authinfo *, void *),
+                        void *arg, const char *newpass)
+{
+	char *newpass_crypt=0;
+	const char *attributes[10], *ldap_attributes[10];
   
-  struct timeval timeout;
+	struct timeval timeout;
 
-  LDAPMessage *result;
-  LDAPMessage *entry;
-  char *filter, *dn;
-  int i, j;
+	LDAPMessage *result;
+	LDAPMessage *entry;
+	char *filter, *dn;
+	int i, j;
 
-struct authinfo auth;
-char *homeDir=0;
-char *mailDir=0;
-char *userPassword=0;
-char *cryptPassword=0;
-char *cn=0;
-uid_t au;
-gid_t ag;
-int rc;
-char *quota=0;
+	struct authinfo auth;
+	char *homeDir=0;
+	char *mailDir=0;
+	char *userPassword=0;
+	char *cryptPassword=0;
+	char *cn=0;
+	uid_t au;
+	gid_t ag;
+	int rc;
+	char *quota=0;
 
 	memset(&auth, 0, sizeof(auth));
 
-  if (ldapopen())	return (1);
+	if (ldapopen())	return (1);
 
-  if ((filter=malloc(strlen(my_ldap.mail)+strlen(user)+
-		(my_ldap.domain ? strlen(my_ldap.domain):0)+
-		sizeof ("(=@)"))) == 0)
-  {
-      perror("malloc");
-      return 1;
-  }
+	if ((filter=malloc(strlen(my_ldap.mail)+strlen(user)+
+			   (my_ldap.domain ? strlen(my_ldap.domain):0)+
+			   sizeof ("(=@)"))) == 0)
+	{
+		perror("malloc");
+		return 1;
+	}
 
-  strcat(strcat(strcat(strcpy(filter, "("), my_ldap.mail), "="), user);
-  if ( my_ldap.domain && my_ldap.domain[0] && strchr(user, '@') == 0 )
-     strcat(strcat(filter, "@"), my_ldap.domain);
-  strcat(filter, ")");
+	strcat(strcat(strcat(strcpy(filter, "("), my_ldap.mail), "="), user);
+	if ( my_ldap.domain && my_ldap.domain[0] && strchr(user, '@') == 0 )
+		strcat(strcat(filter, "@"), my_ldap.domain);
+	strcat(filter, ")");
 
-  timeout.tv_sec=my_ldap.timeout;
-  timeout.tv_usec=0;
+	timeout.tv_sec=my_ldap.timeout;
+	timeout.tv_usec=0;
 
 	read_env("LDAP_HOMEDIR", &attributes[0], "", 0, "homeDir");
 	read_env("LDAP_MAILDIR", &attributes[1], "", 0, 0);
@@ -595,98 +696,100 @@ char *quota=0;
 
 	ldap_attributes[j]=0;
 
-  if (ldaperror(ldap_search_st(my_ldap_fp, (char *)my_ldap.basedn,LDAP_SCOPE_SUBTREE,
-		filter, (char **)ldap_attributes, 0, &timeout, &result)
-		!= LDAP_SUCCESS))
-    {
-      free(filter);
+	if (ldaperror(ldap_search_st(my_ldap_fp,
+				     (char *)my_ldap.basedn,LDAP_SCOPE_SUBTREE,
+				     filter, (char **)ldap_attributes, 0,
+				     &timeout, &result)
+		      != LDAP_SUCCESS))
+	{
+		free(filter);
 
-	if (my_ldap_fp)	return (1);
-	return (-1);
-    }
+		if (my_ldap_fp)	return (1);
+		return (-1);
+	}
 
-   free(filter);
+	free(filter);
 
-  /* If we are more than one result, reject */
-  if (ldap_count_entries(my_ldap_fp,result)!=1)
-    {
-      ldap_msgfree(result);
-      return -1;
-    }
+	/* If we are more than one result, reject */
+	if (ldap_count_entries(my_ldap_fp,result)!=1)
+	{
+		ldap_msgfree(result);
+		return -1;
+	}
 #if DEBUG_LDAP
-  syslog(LOG_DAEMON|LOG_CRIT,"Nombre de rsulat:    %d\n",ldap_count_entries(my_ldap_fp,result));
+	syslog(LOG_DAEMON|LOG_CRIT,"Nombre de rsulat:    %d\n",ldap_count_entries(my_ldap_fp,result));
 #endif
 
-  dn = ldap_get_dn(my_ldap_fp, result);
+	dn = ldap_get_dn(my_ldap_fp, result);
 
 #if DEBUG_LDAP
-  syslog(LOG_DAEMON|LOG_CRIT,"DN:    %s\n",dn);
+	syslog(LOG_DAEMON|LOG_CRIT,"DN:    %s\n",dn);
 #endif
 
-  if (dn == NULL) 
-    {
-      ldap_perror(my_ldap_fp, "ldap_get_dn");
-      return -1;
-    }
+	if (dn == NULL) 
+	{
+		ldap_perror(my_ldap_fp, "ldap_get_dn");
+		return -1;
+	}
 
-  /* Get the pointer on this result */
-  entry=ldap_first_entry(my_ldap_fp,result);
-  if (entry==NULL)
-    {
-      ldap_perror(my_ldap_fp,"ldap_first_entry");
-      free(dn);
-      return -1;
-    }
+	/* Get the pointer on this result */
+	entry=ldap_first_entry(my_ldap_fp,result);
+	if (entry==NULL)
+	{
+		ldap_perror(my_ldap_fp,"ldap_first_entry");
+		free(dn);
+		return -1;
+	}
 
 #if DEBUG_LDAP
-  syslog(LOG_DAEMON|LOG_CRIT,"after ldap_first_entry\n");
+	syslog(LOG_DAEMON|LOG_CRIT,"after ldap_first_entry\n");
 #endif
-  /* Copy the directory and the password into struct */
-  copy_value(my_ldap_fp,entry,attributes[0],&homeDir, user);
-  if (attributes[1])
-	  copy_value(my_ldap_fp,entry,attributes[1],&mailDir, user);
-  copy_value(my_ldap_fp,entry,attributes[2],&cn, user);
-  if (attributes[3])
-	  copy_value(my_ldap_fp,entry,attributes[3],&userPassword, user);
-  if (attributes[4])
-	  copy_value(my_ldap_fp,entry,attributes[4],&cryptPassword, user);
+	/* Copy the directory and the password into struct */
+	copy_value(my_ldap_fp,entry,attributes[0],&homeDir, user);
+	if (attributes[1])
+		copy_value(my_ldap_fp,entry,attributes[1],&mailDir, user);
+	copy_value(my_ldap_fp,entry,attributes[2],&cn, user);
+	if (attributes[3])
+		copy_value(my_ldap_fp,entry,attributes[3],&userPassword, user);
+	if (attributes[4])
+		copy_value(my_ldap_fp,entry,attributes[4],&cryptPassword, user);
 
 	au=my_ldap.uid;
 	ag=my_ldap.gid;
-  if (attributes[5])
-  {
-	char *p=0;
-	unsigned long n;
+	if (attributes[5])
+	{
+		char *p=0;
+		unsigned long n;
 
-	copy_value(my_ldap_fp, entry, attributes[5], &p, user);
-	if (!p)	return (0);
+		copy_value(my_ldap_fp, entry, attributes[5], &p, user);
+		if (!p)	return (0);
 
-
-	if (sscanf(p, "%lu", &n) > 0)
-		au= (uid_t)n;
-	free(p);
+		if (sscanf(p, "%lu", &n) > 0)
+			au= (uid_t)n;
+		free(p);
 #if DEBUG_LDAP
-	syslog(LOG_DAEMON|LOG_CRIT,"au= %d\n",au);
+		syslog(LOG_DAEMON|LOG_CRIT,"au= %d\n",au);
 #endif
-  }
+	}
 
-  if (attributes[6])
-  {
-	char *p=0;
-	unsigned long n;
+	if (attributes[6])
+	{
+		char *p=0;
+		unsigned long n;
 
-	copy_value(my_ldap_fp, entry, attributes[6], &p, user);
-	if (!p)	return (0);
+		copy_value(my_ldap_fp, entry, attributes[6], &p, user);
+		if (!p)	return (0);
 
-	if (sscanf(p, "%lu", &n) > 0)
-		ag= (gid_t)n;
-	free(p);
+		if (sscanf(p, "%lu", &n) > 0)
+			ag= (gid_t)n;
+		free(p);
 #if DEBUG_LDAP
-	syslog(LOG_DAEMON|LOG_CRIT,"ag= %d\n",ag);
+		syslog(LOG_DAEMON|LOG_CRIT,"ag= %d\n",ag);
 #endif
-  }
-  if (attributes[8])
-	  copy_value(my_ldap_fp,entry,attributes[8],&quota, user);
+	}
+
+	if (attributes[8])
+		copy_value(my_ldap_fp,entry,attributes[8],&quota, user);
 
 	auth.sysusername=user;
 	auth.sysuserid= &au;
@@ -709,16 +812,17 @@ char *quota=0;
 
 	if (au == 0 || ag == 0)
 	{
-		syslog(LOG_DAEMON|LOG_CRIT, "authlib: refuse to authenticate %s: uid=%d, gid=%d\n", user, au, ag);
+		syslog(LOG_DAEMON|LOG_CRIT,
+		       "authlib: refuse to authenticate %s: uid=%d, gid=%d\n",
+		       user, au, ag);
 		rc= 1;
 	}
 
-
 	if (pass)
 	{
 		if (my_ldap.authbind) 
 		{
-		LDAP *bindp=ldapconnect();
+			LDAP *bindp=ldapconnect();
 
 			if (!bindp)
 				rc=1;
@@ -737,6 +841,13 @@ char *quota=0;
 				}
 				ldap_unbind(bindp);
 			}
+			if (rc == 0 && newpass)
+			{
+				if ((newpass_crypt=authcryptpasswd(newpass,
+								   NULL))
+				    == 0)
+					rc= -1;
+			}
 		}
 		else
 		{
@@ -749,20 +860,75 @@ char *quota=0;
 			{
 			const char *p=auth.passwd;
 
-				if (p && strncmp(p, "{crypt}", 7) == 0)
+				if (p && strncasecmp(p, "{crypt}", 7) == 0)
 					p += 7; /* For authcheckpassword */
 
 				if (!p || authcheckpassword(pass, p))
 					rc= -1;
 			}
+
+			if (rc == 0 && newpass && auth.passwd)
+			{
+				if ((newpass_crypt=authcryptpasswd(newpass,
+								   auth.passwd)
+				     ) == 0)
+					rc= -1;
+			}
 		}
         }
+
+	if (rc == 0 && newpass)
+	{
+		LDAPMod *mods[3];
+		int mod_index=0;
+
+		LDAPMod mod_clear, mod_crypt;
+		char *mod_clear_vals[2], *mod_crypt_vals[2];
+
+		if (attributes[3])
+		{
+			mods[mod_index]= &mod_clear;
+			mod_clear.mod_op=LDAP_MOD_REPLACE;
+			mod_clear.mod_type=(char *)attributes[3];
+			mod_clear.mod_values=mod_clear_vals;
+
+			mod_clear_vals[0]=(char *)newpass;
+			mod_clear_vals[1]=NULL;
+			++mod_index;
+		}
+
+		if (attributes[4] && newpass_crypt)
+		{
+			mods[mod_index]= &mod_crypt;
+			mod_crypt.mod_op=LDAP_MOD_REPLACE;
+			mod_crypt.mod_type=(char *)attributes[4];
+			mod_crypt.mod_values=mod_crypt_vals;
+
+			mod_crypt_vals[0]=newpass_crypt;
+			mod_crypt_vals[1]=NULL;
+			++mod_index;
+		}
+		if (mod_index == 0)
+			rc= -1;
+		else
+		{
+			mods[mod_index]=0;
+
+			if (ldap_modify_s(my_ldap_fp, dn, mods))
+			{
+				rc= -1;
+			}
+		}
+	}
+
+	if (newpass_crypt)
+		free(newpass_crypt);
 	free (dn);
 #if DEBUG_LDAP
 	syslog(LOG_DAEMON|LOG_CRIT,"before callback rc=%d\n",rc);
 #endif
 
-	if (rc == 0)
+	if (rc == 0 && callback)
 		rc= (*callback)(&auth, arg);
 #if DEBUG_LDAP
 	syslog(LOG_DAEMON|LOG_CRIT,"after callback rc=%d\n",rc);
