@@ -27,7 +27,7 @@
 #include	"maildir/maildirgetquota.h"
 #include	"maildir/maildirquota.h"
 
-static const char rcsid[]="$Id: fetch.c,v 1.15 2001/02/20 12:53:52 mrsam Exp $";
+static const char rcsid[]="$Id: fetch.c,v 1.16 2001/08/31 01:39:49 mrsam Exp $";
 
 unsigned long header_count=0, body_count=0;	/* Total transferred */
 
@@ -621,6 +621,8 @@ off_t cache_phys_chars;
 		{
 			if (ismsgrfc822)
 			{
+				const char *ct, *dummy;
+
 				if (mimep->firstpart == 0)
 				{
 					/* Not a multipart, n must be 1 */
@@ -631,6 +633,15 @@ off_t cache_phys_chars;
 					continue;
 				}
 				ismsgrfc822=0;
+
+				rfc2045_mimeinfo(mimep, &ct,
+						 &dummy,
+						 &dummy);
+
+				if (ct && strcasecmp(ct, "message/rfc822"
+						     ) == 0)
+					ismsgrfc822=1;
+				/* The content is another message/rfc822 */
 			}
 
 			mimep=mimep->firstpart;
