@@ -75,7 +75,7 @@
 
 #define	SHARED	"shared"
 
-static const char rcsid[]="$Id: imapd.c,v 1.64 2001/06/07 02:49:25 mrsam Exp $";
+static const char rcsid[]="$Id: imapd.c,v 1.66 2001/08/25 21:59:26 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern void fetchflags(unsigned long);
@@ -1397,9 +1397,27 @@ int	uid=0;
 	char	*mailbox;
 	struct	imapflags flags;
 	time_t	timestamp=0;
-	struct	imaptoken *tok;
+	struct	imaptoken *tok=nexttoken_nouc();
 
-		mailbox=parse_mailbox_error(tag, (tok=nexttoken_nouc()), 0, 1);
+		if (tok->tokentype != IT_NUMBER &&
+			tok->tokentype != IT_ATOM &&
+			tok->tokentype != IT_QUOTED_STRING)
+			return (-1);
+
+		mailbox=valid_mailbox_name(tok->tokenbuf, 0);
+		if(mailbox && access(mailbox, 0)) {
+		      writes(tag);
+		      writes(" NO [TRYCREATE] Must create mailbox before append\r\n");
+		      free(mailbox);
+		      return (0);
+		}
+
+		if (mailbox)
+		{
+			free(mailbox);
+		}
+
+		mailbox=parse_mailbox_error(tag, tok, 0, 1);
 		if ( mailbox == 0)
 			return (0);
 		if (current_mailbox &&
@@ -1775,7 +1793,6 @@ int	uid=0;
 	{
 	char	*mailbox, *new_mailbox;
 	char	*p;
-	int	ishier=0;
 
 		curtoken=nexttoken_nouc();
 
@@ -1789,10 +1806,7 @@ int	uid=0;
 		}
 
 		if ((p=strrchr(curtoken->tokenbuf, HIERCH))  && p[1] == 0)
-		{
-			ishier=1;
 			*p=0;
-		}
 
 		mailbox=valid_mailbox_name(curtoken->tokenbuf, 0);
 		if ( mailbox == 0)
@@ -1849,9 +1863,26 @@ int	uid=0;
 		fetch_free_cache();
 		if (strcmp(mailbox, "."))
 		{
-			if (is_reserved(mailbox) ||
-				is_reserved(new_mailbox) ||
-				hier_rename(mailbox, new_mailbox, ishier))
+			if (is_reserved(mailbox) || is_reserved(new_mailbox))
+			{
+				free(mailbox);
+				free(new_mailbox);
+				writes(tag);
+				writes(" NO Cannot rename this folder.\r\n");
+				return (0);
+			}
+
+			/* rename recursive: all subfolder */
+			if (hier_rename(mailbox, new_mailbox, 1))
+			{
+				free(mailbox);
+				free(new_mailbox);
+				writes(tag);
+				writes(" NO Cannot rename this folder.\r\n");
+				return (0);
+			}
+			/* rename the base folder */
+			if (hier_rename(mailbox, new_mailbox, 0))
 			{
 				free(mailbox);
 				free(new_mailbox);
