@@ -55,7 +55,7 @@
 #include	"imapwrite.h"
 #include	"imapd.h"
 
-static const char rcsid[]="$Id: imapscanclient.c,v 1.13 2000/11/14 13:16:36 mrsam Exp $";
+static const char rcsid[]="$Id: imapscanclient.c,v 1.15 2001/09/10 11:23:53 mrsam Exp $";
 
 static int do_imapscan_maildir2(struct imapscaninfo *, const char *, int);
 
@@ -117,9 +117,19 @@ char	*p=malloc(strlen(dir)+strlen(new_or_cur)+2);
 	return (p);
 }
 
+static int fnamcmp(const char *a, const char *b)
+{
+	long ai, bi;
+	ai = atol(a);
+	bi = atol(b);
+	if(ai - bi)
+		return ai - bi;
+	return strcmp(a, b);
+}
+
 static int sort_by_filename(struct tempinfo **a, struct tempinfo **b)
 {
-	return (strcmp( (*a)->filename, (*b)->filename));
+	return (fnamcmp( (*a)->filename, (*b)->filename));
 }
 
 static int sort_by_filename_status(struct tempinfo **a, struct tempinfo **b)
@@ -135,7 +145,7 @@ static int sort_by_filename_status(struct tempinfo **a, struct tempinfo **b)
 	if ( (*a)->found )	return (-1);
 	if ( (*b)->found )	return (1);
 
-	return (strcmp( (*a)->filename, (*b)->filename));
+	return (fnamcmp( (*a)->filename, (*b)->filename));
 }
 
 /* Binary search on an array of tempinfos which is sorted by filenames */
@@ -149,7 +159,7 @@ int	rc;
 	while (lo < hi)
 	{
 		mid=(hi+lo)/2;
-		rc=strcmp( a[mid]->filename, filename);
+		rc=fnamcmp( a[mid]->filename, filename);
 		if (rc < 0)
 		{
 			lo=mid+1;
