@@ -1,5 +1,5 @@
 /*
-** Copyright 1998 - 1999 Double Precision, Inc.
+** Copyright 1998 - 2000 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 
@@ -61,12 +61,13 @@
 
 #include	"maildir/maildirmisc.h"
 
-static const char rcsid[]="$Id: mailboxlist.c,v 1.7 2000/06/14 16:42:36 mrsam Exp $";
+static const char rcsid[]="$Id: mailboxlist.c,v 1.11 2000/08/20 19:01:34 mrsam Exp $";
 
 static const char hierchs[]={HIERCH, 0};
 
 extern char *set_namespace_options(char *);
 extern char *valid_mailbox_name(const char *, int);
+extern char *decode_valid_mailbox(const char *, int);
 /*
 		LIST MAILBOXES
 */
@@ -151,22 +152,13 @@ int	nullname= *name == 0;
 
 static int match_mailbox(char *, char *);
 
-/* Check if INBOX has any new messages */
+/* Check if a folder has any new messages */
 
-static int hasnewmsgs()
+static int hasnewmsgs2(const char *dir)
 {
-DIR	*dirp=opendir("new");
+DIR	*dirp=opendir(dir);
 struct	dirent *de;
 
-	while (dirp && (de=readdir(dirp)) != 0)
-	{
-		if (de->d_name[0] == '.')	continue;
-		closedir(dirp);
-		return (1);
-	}
-	if (dirp)	closedir(dirp);
-
-	dirp=opendir("cur");
 	while (dirp && (de=readdir(dirp)) != 0)
 	{
 	char	*p;
@@ -181,7 +173,39 @@ struct	dirent *de;
 		}
 	}
 	if (dirp)	closedir(dirp);
+	return (0);
+}
 
+static int hasnewmsgs(const char *folder)
+{
+char *dir=decode_valid_mailbox(folder, 0);
+char *subdir;
+
+	if (!dir)	return (0);
+
+	if (is_sharedsubdir(dir))
+		maildir_shared_sync(dir);
+
+	subdir=malloc(strlen(dir)+sizeof("/cur"));
+	if (!subdir)	write_error_exit(0);
+	strcat(strcpy(subdir, dir), "/new");
+	if (hasnewmsgs2(subdir))
+	{
+		free(subdir);
+		free(dir);
+		return (1);
+	}
+
+	strcat(strcpy(subdir, dir), "/cur");
+	if (hasnewmsgs2(subdir))
+	{
+		free(subdir);
+		free(dir);
+		return (1);
+	}
+
+	free(subdir);
+	free(dir);
 	return (0);
 }
 
@@ -273,6 +297,28 @@ char	*p=malloc(strlen(n)+sizeof("shared."));
 	free(p);
 }
 
+static void list_subscribed(char *hier,
+	int *found_hier, struct hierlist **folders,
+	struct hierlist **hierarchies)
+{
+char	buf[BUFSIZ];
+FILE	*fp;
+
+	fp=fopen(SUBSCRIBEFILE, "r");
+	if (fp)
+	{
+		while (fgets(buf, sizeof(buf), fp) != 0)
+		{
+		char *q=strchr(buf, '\n');
+
+			if (q)	*q=0;
+			folder_entry(buf, hier, found_hier,
+				folders, hierarchies);
+		}
+		fclose(fp);
+	}
+}
+
 static void do_mailbox_list(const char *tag, char *qq, int isnullname,
 	const char *cmd)
 {
@@ -286,6 +332,20 @@ char	*namesp=0;
 struct list_sharable_info shared_info;
 char	*newqq=0;
 
+const char *obsolete;
+int check_all_folders=0;
+
+int do_lsub=strcmp(cmd, "LIST") != 0;
+
+	obsolete=getenv("IMAP_CHECK_ALL_FOLDERS");
+	if (obsolete && atoi(obsolete))
+		check_all_folders=1;
+
+	obsolete=getenv("IMAP_OBSOLETE_CLIENT");
+
+	if (obsolete && atoi(obsolete) == 0)
+		obsolete=0;
+
 	if (*qq == '#')
 	{
 	char	*p=qq;
@@ -321,6 +381,9 @@ char	*newqq=0;
 			strcmp(de->d_name, "..") == 0)
 			continue;
 
+		if (do_lsub)
+			continue;	/* Skip this whole thing for LSUBs */
+
 		if ((p=malloc(strlen(de->d_name)+sizeof("INBOX."))) == 0)
 					/* A bit too much, that's OK */
 			write_error_exit(0);
@@ -354,7 +417,7 @@ char	*newqq=0;
 	shared_info.folders= &folders;
 	shared_info.hierarchies= &hierarchies;
 
-	if (strcmp(cmd, "LIST") == 0)
+	if (!do_lsub)
 	{
 		/* List sharable maildirs */
 
@@ -362,6 +425,8 @@ char	*newqq=0;
 	}
 	else
 	{
+		list_subscribed(qq, isnullname ? &found_hier:0,
+			&folders, &hierarchies);
 
 		/* List shared folders */
 
@@ -377,8 +442,8 @@ char	*newqq=0;
 
 		is_interesting= -1;
 
-		if (strcmp(hp->hier, "INBOX") == 0)
-			is_interesting=hasnewmsgs();
+		if (strcmp(hp->hier, "INBOX") == 0 || check_all_folders)
+			is_interesting=hasnewmsgs(hp->hier);
 
 		writes("* ");
 		writes(cmd);
@@ -398,10 +463,21 @@ char	*newqq=0;
 			(d=search_hier(hierarchies, hp->hier)) == 0)
 		{
 			writes(sep);
-			writes("\\Noinferiors");
+
+			if (obsolete)
+				writes("\\Noinferiors");
+			else
+				writes("\\HasNoChildren");
 		}
 		else
+		{
 			d->flag=1;
+			if (!obsolete)
+			{
+				writes(sep);
+				writes("\\HasChildren");
+			}
+		}
 
 		writes(") ");
 
@@ -438,7 +514,15 @@ char	*newqq=0;
 			{
 				writes("* ");
 				writes(cmd);
-				writes(" (\\Noselect) ");
+
+				if (obsolete)
+				{
+					writes(" (\\Noselect) ");
+				}
+				else
+				{
+					writes(" (\\Noselect \\HasChildren) ");
+				}
 
 				if (flat_folders && strchr(hp->hier, HIERCH))
 					writes("NIL");
