@@ -28,7 +28,7 @@
 #include	"imaptoken.h"
 #include	"imapscanclient.h"
 
-static const char rcsid[]="$Id: search.c,v 1.17 2001/03/23 05:13:43 mrsam Exp $";
+static const char rcsid[]="$Id: search.c,v 1.18 2001/08/31 01:39:49 mrsam Exp $";
 
 extern time_t rfc822_parsedt(const char *);
 extern struct imapscaninfo current_mailbox_info;
@@ -451,8 +451,10 @@ int	rc, rc2;
         case search_reverse:
         case search_size:
         case search_to:
-		if (si->value < 0)	return (-1);
-		return (search_evaluate(si->a));
+		rc = search_evaluate(si->a);
+		if (rc == 0) return 0;
+		if (si->value < 0)  return (-1);
+		break;
 	case search_not:
 		rc=search_evaluate(si->a);
 		if (rc >= 0)	rc= 1-rc;
