@@ -45,7 +45,7 @@
 #include	<time.h>
 #include	<numlib/numlib.h>
 
-static const char rcsid[]="$Id: maildirquota.c,v 1.3 2000/05/24 03:42:00 mrsam Exp $";
+static const char rcsid[]="$Id: maildirquota.c,v 1.6 2001/07/24 12:27:13 mrsam Exp $";
 
 /* Read the maildirsize file */
 
@@ -131,9 +131,11 @@ static int statsubdir(const char *, const char *, time_t *);
 				maildir -- NOT IMPLEMENTED */
 #define	MDQUOTA_COUNT	'C'	/* Total number of messages in maildir */
 
-static int qcalc(off_t s, unsigned n, const char *quota)
+static int qcalc(off_t s, unsigned n, const char *quota, int *percentage)
 {
 unsigned long i;
+int	spercentage=0;
+int	npercentage=0;
 
 	errno=ENOSPC;
 	while (quota && *quota)
@@ -148,24 +150,66 @@ unsigned long i;
 			i=i*10 + (*quota++ - '0');
 		switch (*quota)	{
 		default:
-			if (i < s)	return (-1);
+			if (i < s)
+			{
+				*percentage=100;
+				return (-1);
+			}
+			spercentage = i ? s * 100 / i:100;
 			break;
 		case 'C':
-			if (i < n)	return (-1);
+			if (i < n)
+			{
+				*percentage=100;
+				return (-1);
+			}
+			npercentage = i ? n * (unsigned long)100 / i:100;
 			break;
 		}
 	}
+	*percentage = spercentage > npercentage ? spercentage:npercentage;
 	return (0);
 }
 
 static int	doaddquota(const char *, int, const char *, long, int, int);
 
+static int docheckquota(const char *dir,
+	int *maildirsize_fdptr,
+	const char *quota_type,
+	long xtra_size,
+	int xtra_cnt, int *percentage);
+
+
 int maildir_checkquota(const char *dir,
 	int *maildirsize_fdptr,
 	const char *quota_type,
 	long xtra_size,
 	int xtra_cnt)
 {
+int	dummy;
+
+	return (docheckquota(dir, maildirsize_fdptr, quota_type,
+		xtra_size, xtra_cnt, &dummy));
+}
+
+int maildir_readquota(const char *dir, const char *quota_type)
+{
+int	percentage=0;
+int	fd=-1;
+
+	(void)docheckquota(dir, &fd, quota_type, 0, 0, &percentage);
+	if (fd >= 0)
+		close(fd);
+	return (percentage);
+}
+
+static int docheckquota(const char *dir,
+	int *maildirsize_fdptr,
+	const char *quota_type,
+	long xtra_size,
+	int xtra_cnt,
+	int *percentage)
+{
 char	*checkfolder=(char *)malloc(strlen(dir)+sizeof("/maildirfolder"));
 char	*newmaildirsizename;
 struct stat stat_buf;
@@ -185,8 +229,8 @@ struct dirent *de;
 	if (stat(checkfolder, &stat_buf) == 0)	/* Go to parent */
 	{
 		strcat(strcpy(checkfolder, dir), "/..");
-		n=maildir_checkquota(checkfolder, maildirsize_fdptr,
-			quota_type, xtra_size, xtra_cnt);
+		n=docheckquota(checkfolder, maildirsize_fdptr,
+			quota_type, xtra_size, xtra_cnt, percentage);
 		free(checkfolder);
 		return (n);
 	}
@@ -199,7 +243,7 @@ struct dirent *de;
 		&maildirsize_nlines, &stat_buf) == 0)
 	{
 		n=qcalc(maildirsize_size+xtra_size, maildirsize_cnt+xtra_cnt,
-			quota_type);
+			quota_type, percentage);
 
 		if (n == 0)
 		{
@@ -320,7 +364,7 @@ struct dirent *de;
 	}
 
 	return (qcalc(maildirsize_size+xtra_size, maildirsize_cnt+xtra_cnt,
-		quota_type));
+		quota_type, percentage));
 }
 
 int	maildir_addquota(const char *dir, int maildirsize_fd,
